<?php
/**
 * Aggregate core
 *  
 *
 * @author Michael Tyson
 * @version 20090303
 * @copyright Michael Tyson http://michael.tyson.id.au 2008
 * @package aggregate
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 **/


/**
 * Configuration
 *
 *  Must define an array 'feeds' which is an array of associative arrays.
 *  Each feed definition consists of:
 *
 *      url         The URL of the feed
 *      trigger     A regular expression (as in first argument to preg_match) that must be matched by the 
 *                  title/content, or an associative array, where keys are properties of the RSS item to search, 
 *                  and values are regular expressions to match ('and' relationship).  This field is optional.
 *      filter      Expression similar in syntax to s/// match/replace syntax found in Sed and Perl, among others.
 *                  Can use arbitrary separators; first item is the search, second item is the replacement, and third
 *                  item, if present, is the flags to apply. Separate multiple expressions with semi-colons.
 *                  Expression will be used to filter the title/content of the item.  Can also be an associative array, 
 *                  as for the 'trigger' field.
 *                  eg. "@Related posts:.*@@" will remove all text from 'Related posts:'.
 *                  Common fields: title, description, atom_content
 *                  
 */

global $feeds;

// Sample entries, to take items with category 'Loopy' from an RSS feed, and twitter entries with 'loopy' mentioned
$feeds = array(
	array("url" => "http://michael.tyson.id.au/tag/loopy/feed/",
			"trigger" => array("category" => "/Loopy/"),
			"filter" => "/Related posts:.*//;/\[[^\]]+\]//"),
	array("url" => "http://twitter.com/statuses/user_timeline/14104615.rss",
			"trigger" => "/#loopy/i;!/@[a-zA-Z0-9]/",
			"filter" => array(
			        "description" => ($filter="/^MichaelTyson: //;@http://[^\s]+@<a href=\"$0\">$0</a>@"), // Remove name and add links
			        "atom_content" => $filter,
			        "title" => $filter.";/(^.{20,40})\s.*/$1...")) // Same as above, but also truncate the title
);

// Optional time offset, in seconds: Will modify the 'date_timestamp' field
define("TIME_OFFSET", 0 * 60 * 60);



/******************************************************************************************/
/******************************************************************************************/





/**
 * Get RSS items
 *
 * @return Array of RSS items
 */
function aggregate_items() {
	include("contrib/magpie/rss_fetch.inc");
	global $feeds;
	$items = array();
	
	$relationship = "and";
	
	// Fetch items
	foreach ( $feeds as $feed ) {
		$rss = fetch_rss($feed['url']);
	    
		foreach ( $rss->items as $item ) {
					
			// Check triggers
			if ( $feed['trigger'] ) {
			    if ( !is_array($feed['trigger']) ) {
			        $feed['trigger'] = array('title' => $feed['trigger'], 'atom_content' => $feed['trigger'], 'description' => $feed['trigger']);
			        $relationship = "or";
                }
				$match = ($relationship=="and" ? true : false);
				foreach ( $feed['trigger'] as $key => $value ) {
				    $expressions = explode(";", str_replace("\;", "##ESCAPED##", $value));
				    $expressions = array_map(create_function('$expression', 'return str_replace("##ESCAPED##", ";", $expression);'), $expressions);
				    $result = true;
				    foreach ( $expressions as $value ) {
    				    $negate = false;
    				    if ( $value{0} == "!" ) {
    				        $negate = true;
    				        $value = substr($value, 1);
    				    }
    					$val = preg_match($value, $item[$key]);
    					if ( $negate ) $val = !$val;
    					$result = $result && $val;
					}
					$match = ($relationship=="and" ? $match && $result : $match || $result);
				}
				
				if ( !$match ) continue;
			}
		
		    // Apply filters
			if ( $feed['filter'] ) {
			    if ( !is_array($feed['filter']) )
			        $feed['filter'] = array('title' => $feed['filter'], 'atom_content' => $feed['filter'], 'description' => $feed['filter']);
			    foreach ( $feed['filter'] as $key => $value ) {
    				$filters = explode(';', $value);
    				foreach ( $filters as $filter ) {
    					$components = explode($filter{0}, $filter);
    					$item[$key] = preg_replace($filter{0}.$components[1].$filter{0}.$components[3],
    											$components[2],
    											$item[$key]);
    				}
				}
			}
		
	        // Apply time offset
			if (defined("TIME_OFFSET")) 
			    $item['date_timestamp'] += TIME_OFFSET;
			    
			// A bug in the magpie RSS parser concatenates categories together. This workaround splits them into an array, assuming the categories
			// start with a capital letter. If this isn't the case, you may need to find an alternate solution.
			$item['categories'] = array_filter(preg_split("@([A-Z][a-z]*)@", $item['categories'], -1, PREG_SPLIT_DELIM_CAPTURE), create_function('$a', 'return $a != "";'));
			
			// Make sure there's no html in the title
			$item['title'] = strip_tags($item['title']);
			
			$items[] = $item;
		}
	}

	// Sort items by date
	usort($items, create_function('$a, $b', 'return $a["date_timestamp"] < $b["date_timestamp"];'));
	
	return $items;
}

/**
 * Render items
 *
 * @param array $items Array of RSS items
 */
function render_aggregate_items($items) {
	
	foreach ( $items as $item ) {
		?>
		<div class="item">
			<div class="date"><a href="<?php echo $item['link'] ?>"><?php echo date("j M, g:i a", $item['date_timestamp']) ?></a></div>
		
			<?php echo $item['description'] ?>
			
			<div class="readmore">
				<a href="<?php echo $item['link'] ?>">Read more</a>
			</div>
		</div>
		<?php
	}
}

?>